# HSM-I18N

基于 `vue-i18n` 封装的动态国际化组件，封装主要是添加了一些初始化逻辑和加载语言包的逻辑

## 使用步骤

- 1、安装
`pnpm i @hsmos/hsm-i18n`

- 2、实例化

```ts
// /src/utils/i18n.ts
import HsmI18n from '@hsmos/hsm-i18n'
// 默认读取当前项目下 /langs 中的 config.json 和 语言包 文件，默认语言为 zh-CN ，默认存储在 localStorage 中的国际化 key 为 --lang
const options = {
  langKey: '--lang', // 在 localstorage 中存储的当前语言 key
  defaultLang: 'zh-CN',
  resourceUrl: '/langs'
}
const hsmI18n = new HsmI18n(options)
export default hsmI18n
```

- 3、注册

```ts
// /src/main.ts
import hsmI18n from '@/utils/i18n'
app.use(hsmI18n.i18n)
```

- 4、加载语言

在路由拦截中判断 hsmI18n.isLoaded 是否已加载，如果已加载直接 next，如果未加载 await init 方法加载语言包。

```ts
// router.ts
router.beforeEach(async (to, from, next) => {
  document.title = to.meta.title
  if (!hsmI18n.isLoaded) {
    await hsmI18n.init()
  }
  next()
})
```

- 5、切换语言

```ts
import hsmI18n from '@/utils/i18n'
await hsmI18n.setLang('en-US')
```

- 6、html 中使用

```xml
<p class="system">
  {{ $t('login.text.WelcomeToTheIntelligentSoftware') }}
</p>
```

- 7、ts/js 中使用

```vue
<script lang="ts" setup>
import hsmI18n from '@/utils/i18n'
const { t } = hsmI18n.global

t('message.validateError.cannotEmptyMessage')
</script>
```

```ts
import hsmI18n from '@/utils/i18n'
const { locale } = hsmI18n.global

// 监听语言变化
watch(
  () => locale.value,
  (curLang: any) => {
    console.log('当前语言：', curLang)
  }
)
```

## 注意事项

- vue-i18n 的 t 函数，在 javascript 中使用时，切换不会自动更新。尽量把国际化做在 html 中，不要在 ts/js 中使用 t 函数。如果确需使用，需要监听国际化的变化重新执行 t 函数。像ElMessage 中实时调用 t 函数没有问题。有点绕，得好好理解一下

## 国际化资源文件目录结构

/langs                # 语言包根路径
  ├─config.json       # 语言包配置文件
  ├─zh-CN.json        # 中文语言包
  └─en-US.json        # 英文语言包

## config.json 文件说明

```json
{
  "langs": [
    {
      "name": "中文简体",
      "fileName": "zh-CN"
    },
    {
      "name": "English",
      "fileName": "en-US"
    }
  ]
}
```

表示当前项目有 2 种语言可选择，分别是 `中文简体` 和 `English`，fileName 是语言包在 `resourceUrl` 目录的文件名。

## 推荐 vscode 辅助插件： i18n-ally

对于 json 文件支持度非常高，可以做到在源码中察看结果、修改语言信息、分析翻译覆盖度等。

```json
// vscode 配置文件 setting.json
"i18n-ally.localesPaths": ["app/public/langs"],
"i18n-ally.extract.autoDetect": true,
"i18n-ally.sourceLanguage": "zh-CN",
"i18n-ally.displayLanguage": "zh-CN",
"i18n-ally.sortKeys": false,
"i18n-ally.enabledParsers": ["ts", "json"],
"i18n-ally.enabledFrameworks": ["vue", "vue-sfc"],
"i18n-ally.keystyle": "nested",
"i18n-ally.ignoreFiles": ["**/config.json"],

## 语言包文件名定义规范

语言包文件名需要遵守如下命名规范：

```txt
日本 : ja-JP
秘鲁 : es-PE
巴拿马 : es-PA
波斯尼亚和黑山共和国 : sr-BA
危地马拉 : es-GT
阿拉伯联合酋长国 : ar-AE
挪威 : no-NO
阿尔巴尼亚 : sq-AL
伊拉克 : ar-IQ
也门 : ar-YE
葡萄牙 : pt-PT
塞浦路斯 : el-CY
卡塔尔 : ar-QA
马其顿王国 : mk-MK
瑞士 : de-CH
美国 : en-US
芬兰 : fi-FI
马耳他 : en-MT
斯洛文尼亚 : sl-SI
斯洛伐克 : sk-SK
土耳其 : tr-TR
沙特阿拉伯 : ar-SA
英国 : en-GB
塞尔维亚及黑山 : sr-CS
新西兰 : en-NZ
挪威 : no-NO
立陶宛 : lt-LT
尼加拉瓜 : es-NI
爱尔兰 : ga-IE
比利时 : fr-BE
西班牙 : es-ES
黎巴嫩 : ar-LB
加拿大 : fr-CA
爱沙尼亚 : et-EE
科威特 : ar-KW
塞尔维亚 : sr-RS
美国 : es-US
墨西哥 : es-MX
苏丹 : ar-SD
印度尼西亚 : in-ID
乌拉圭 : es-UY
拉脱维亚 : lv-LV
巴西 : pt-BR
叙利亚 : ar-SY
多米尼加共和国 : es-DO
瑞士 : fr-CH
印度 : hi-IN
委内瑞拉 : es-VE
巴林 : ar-BH
菲律宾 : en-PH
突尼斯 : ar-TN
奥地利 : de-AT
荷兰 : nl-NL
厄瓜多尔 : es-EC
台湾地区 : zh-TW
约旦 : ar-JO
冰岛 : is-IS
哥伦比亚 : es-CO
哥斯达黎加 : es-CR
智利 : es-CL
埃及 : ar-EG
南非 : en-ZA
泰国 : th-TH
希腊 : el-GR
意大利 : it-IT
匈牙利 : hu-HU
爱尔兰 : en-IE
乌克兰 : uk-UA
波兰 : pl-PL
卢森堡 : fr-LU
比利时 : nl-BE
印度 : en-IN
西班牙 : ca-ES
摩洛哥 : ar-MA
玻利维亚 : es-BO
澳大利亚 : en-AU
新加坡 : zh-SG
萨尔瓦多 : es-SV
俄罗斯 : ru-RU
韩国 : ko-KR
阿尔及利亚 : ar-DZ
越南 : vi-VN
黑山 : sr-ME
利比亚 : ar-LY
中国 : zh-CN
台湾：zh-TW
香港 : zh-HK
白俄罗斯 : be-BY
以色列 : iw-IL
保加利亚 : bg-BG
马耳他 : mt-MT
巴拉圭 : es-PY
法国 : fr-FR
捷克共和国 : cs-CZ
瑞士 : it-CH
罗马尼亚 : ro-RO
波多黎哥 : es-PR
加拿大 : en-CA
德国 : de-DE
卢森堡 : de-LU
阿根廷 : es-AR
马来西亚 : ms-MY
克罗地亚 : hr-HR
新加坡 : en-SG
阿曼 : ar-OM
泰国 : th-TH
瑞典 : sv-SE
丹麦 : da-DK
洪都拉斯 : es-HN
```

## 更新日志

- 0.1.1 发布 hsm-i18n 组件
- 0.1.3 提供 global 属性
- 0.1.4 扩展 LangList 类型属性
