import { type I18n, type I18nOptions, createI18n, Composer } from 'vue-i18n'

const DEFAULT_LANG = 'zh-CN'
const HSM_LANG = '--lang'

export interface HsmI18nOptions {
  langKey?: string
  defaultLang?: string
  resourceUrl?: string
}

export interface LangList {
  name: string
  fileName: string
  isLoaded?: boolean
  [key: string]: any
}

export class HsmI18n {
  private _i18n: I18n
  private _langKey = HSM_LANG // 本地缓存的语言设置字段名
  private _lang = DEFAULT_LANG
  private _defaultLang = DEFAULT_LANG
  private _resourceUrl = '/langs'
  private _langs: Array<LangList> = []
  private _isLoadedLang = false

  // step1: 实例化
  constructor(options?: HsmI18nOptions, i18nOptions?: I18nOptions) {
    if (options?.langKey) {
      this._langKey = options.langKey
    }
    if (options?.defaultLang) {
      this._defaultLang = options.defaultLang
    }
    if (options?.resourceUrl) {
      this._resourceUrl = options.resourceUrl
    }
    // 首先检查 localStorage 中保存的 lang ，再检查 浏览器首选语言，最后再设置默认语言
    const cacheLang = localStorage.getItem(this._langKey) as string
    if (cacheLang && cacheLang !== 'null') {
      this._lang = cacheLang
    } else {
      const locale = navigator.language
      if (locale) {
        this._lang = locale
        localStorage.setItem(this._langKey, this._lang)
      }
    }
    // 初始化 vue-i18n
    this._i18n = createI18n(
      Object.assign(
        {
          legacy: false,
          globalInjection: true,
          fallbackLocale: this._defaultLang,
          locale: this._lang,
        },
        i18nOptions
      )
    )
  }

  // step2: 初始化获取配置文件和当前语言包
  async init() {
    if (await this._loadLang(this._lang)) {
      // 设置 vue-i18n 语言设置
      if (typeof this._i18n.global.locale === 'string') {
        this._i18n.global.locale = this._lang
      } else {
        this._i18n.global.locale.value = this._lang
      }
    }
    // 初始化的时候如果当前语言和默认语言不一致，加载一下默认语言
    if (this._lang !== this._defaultLang) {
      await this._loadLang(this._defaultLang)
    }
    this._isLoadedLang = true
  }

  // step3: 修改语言
  async setLang(val: string) {
    if (val !== this._lang) {
      this._lang = val
      this._isLoadedLang = false
      // 请求语言包
      if (await this._loadLang(val)) {
        this._isLoadedLang = true
        // 设置 vue-i18n 语言设置
        if (typeof this._i18n.global.locale === 'string') {
          this._i18n.global.locale = this._lang
        } else {
          this._i18n.global.locale.value = this._lang
        }
        localStorage.setItem(this._langKey, val)
      }
    }
  }

  get i18n() {
    return this._i18n
  }

  get global(): Composer {
    return this._i18n.global as Composer
  }

  // 获取当前语言
  get lang() {
    return this._lang
  }

  // 获取当前语言列表
  get langs() {
    return this._langs
  }

  // 是否已经加载了语言包
  get isLoaded() {
    return this._isLoadedLang
  }

  // 读取 config.json 获取支持的语言列表
  private async _loadLangs() {
    if (!this._langs.length) {
      const resJSON = await fetch(`${this._resourceUrl}/config.json`)
      if (resJSON) {
        const res = await resJSON.json()
        if (res && res.langs && res.langs.length) {
          this._langs = res.langs
        }
      }
    }
    return this._langs
  }

  // 加载语言包
  private async _loadLang(lang: string) {
    // 先检查语言列表中有没有当前设置的语言
    const list = await this._loadLangs()
    const currentLangItem = list.find((v) => v.fileName === lang)
    if (list && list.length && currentLangItem) {
      if (!currentLangItem.isLoaded) {
        const resJSON = await fetch(`${this._resourceUrl}/${lang}.json`)
        if (resJSON) {
          const res = await resJSON.json()
          if (res && this._i18n) {
            this._i18n.global.mergeLocaleMessage(lang, res)
            currentLangItem.isLoaded = true
          }
        }
      }
      return true
    }
    return false
  }
}

export default HsmI18n
